import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { useContext, useMemo, useState, useEffect } from "react";
import { StoreManager } from "./context";
import { path, propOr, noop, diff } from "./utils";

var getStateProps = function getStateProps(_ref) {
  var selectors = _ref.selectors,
      instanceId = _ref.instanceId,
      cache = _ref.cache,
      state = _ref.state,
      props = _ref.props;
  return Object.entries(selectors || {}).reduce(function (acc, _ref2) {
    var _ref3 = _slicedToArray(_ref2, 2),
        key = _ref3[0],
        f = _ref3[1];

    cache[f.id] = cache[f.id] || {};
    return _objectSpread({}, acc, _defineProperty({}, key, f(state, _objectSpread({}, props || {}, {
      instanceId: instanceId
    }), cache)));
  }, {});
}; // eslint-disable-next-line no-unused-vars


export var useModel = function useModel(selectors, actions, _ref4) {
  var children = _ref4.children,
      props = _objectWithoutProperties(_ref4, ["children"]);

  var context = useContext(StoreManager);
  var instanceId = propOr(context.instanceId, "instanceId", props);
  var store = path(["moduleInstances", context.contextId, "store"], context);

  if (!store && process.env.NODE_ENV !== "production") {
    throw Error("No store found for `useModel`. Please use `createContext` in a parent component.");
  }

  var _useState = useState({}),
      _useState2 = _slicedToArray(_useState, 1),
      cache = _useState2[0];

  var iniStateProps = useMemo(function () {
    return getStateProps({
      selectors: selectors,
      instanceId: instanceId,
      cache: cache,
      state: store.getState(),
      props: props
    });
  }, []);

  var _useState3 = useState(iniStateProps),
      _useState4 = _slicedToArray(_useState3, 2),
      stateProps = _useState4[0],
      setStateProps = _useState4[1];

  useEffect(function () {
    if (store) {
      var oldStateProps = stateProps;
      return store.subscribe(function () {
        var newStateProps = getStateProps({
          selectors: selectors,
          instanceId: instanceId,
          cache: cache,
          state: store.getState(),
          props: props
        });

        if (diff(oldStateProps, newStateProps)) {
          oldStateProps = newStateProps;
          setStateProps(newStateProps);
        }
      }, context.renderLevel);
    }

    return noop;
  }, [store, props, instanceId]);
  var actionProps = useMemo(function () {
    return Object.entries(actions || {}).reduce(function (actions, _ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          key = _ref6[0],
          f = _ref6[1];

      if (typeof f !== "function") {
        throw Error("".concat(key, " is not a function"));
      }

      actions[key] = function (payload) {
        var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var error = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
        return store.dispatch(f(payload, _objectSpread({}, props, {
          meta: _objectSpread({
            instanceId: instanceId
          }, props.meta || {})
        }), error));
      };

      return actions;
    }, {});
  }, [instanceId]);
  return useMemo(function () {
    return _objectSpread({}, stateProps, {}, actionProps, {
      instanceId: instanceId
    });
  }, [stateProps, actionProps, instanceId]);
};