"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createModule = exports.addBridge = exports.merge = exports.addLifecycle = exports.addReducer = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _redux = require("redux");

var _utils = require("./utils");

var _constants = require("./constants");

var _middleware = require("./middleware");

var addReducer = function addReducer() {
  return function (module) {
    var dependencies = Object.values(module.dependencies || {});
    var iniState = (0, _utils.mergeAll)([module.iniState].concat((0, _toConsumableArray2["default"])(dependencies.map(function (dep) {
      return dep.iniState || {};
    }))));
    var reducers = (0, _utils.mergeAll)([module.reducers].concat((0, _toConsumableArray2["default"])(dependencies.map(function (dep) {
      return dep.reducers || {};
    }))));
    return (0, _objectSpread3["default"])({}, module, {
      iniState: iniState,
      reducers: reducers,
      reducer: function reducer(state, action) {
        if (state === undefined) {
          return iniState;
        }

        var globalUpdate = (0, _utils.path)(["meta", _constants.GLOBAL_UPDATE], action);

        if (globalUpdate) {
          return (0, _utils.mergeAll)([state, globalUpdate]);
        }

        var selectedReducer = reducers[action.type];

        if (typeof selectedReducer !== "function") {
          return state;
        }

        var instanceId = (0, _utils.getActionInstanceId)(action, (0, _utils.pathOr)( // if the action is not in dependencies then must be an action from the root module
        module.singleton, [action.namespace, "singleton"], module.dependencies));
        var namespace = action.namespace;
        var p = [namespace, instanceId];
        var oldState = (0, _utils.pathOr)((0, _utils.path)([namespace, _constants.DEFAULT_INSTANCE_ID], iniState), p, state);
        var newState = selectedReducer(oldState, action);

        if (newState !== oldState) {
          return (0, _utils.mergeAll)([state, (0, _defineProperty2["default"])({}, namespace, (0, _utils.mergeAll)([state[namespace], (0, _defineProperty2["default"])({}, instanceId, (0, _utils.mergeAll)([oldState, newState]))]))]);
        }

        return state;
      }
    });
  };
};

exports.addReducer = addReducer;

var defaultCompose = function defaultCompose() {
  return _redux.compose;
};

var composeEnhancers = process.env.NODE_ENV !== "production" && window && // eslint-disable-next-line no-underscore-dangle
window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ || defaultCompose;
var created = Symbol("created");

var addLifecycle = function addLifecycle() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (module) {
    return (0, _objectSpread3["default"])({}, module, {
      constructor: function constructor(_ref3) {
        var _ref3$persist = _ref3.persist,
            persist = _ref3$persist === void 0 ? true : _ref3$persist,
            moduleInstances = _ref3.moduleInstances,
            contextId = _ref3.contextId;
        var moduleInstance = moduleInstances[contextId];
        var globalInstance = contextId === _constants.GLOBAL_CONTEXT_ID ? moduleInstance : moduleInstances[_constants.GLOBAL_CONTEXT_ID];

        if (!moduleInstance[created] || !persist) {
          moduleInstance[created] = true;
          var middleware = Object.values(moduleInstance.middleware || {});

          if ((0, _utils.applyBridgeMiddleware)({
            moduleInstance: moduleInstance,
            globalInstance: globalInstance
          })) {
            //create bridge middleware
            moduleInstance.bridgeMiddleware = (0, _middleware.makeBridgeMiddleware)({
              moduleInstance: moduleInstance,
              globalInstance: globalInstance
            });
            middleware.push(moduleInstance.bridgeMiddleware);
          }

          var iniState = moduleInstance.iniState;

          if (contextId !== _constants.GLOBAL_CONTEXT_ID && (0, _utils.prop)("store", globalInstance)) {
            iniState = (0, _utils.mergeAll)([moduleInstance.iniState, (0, _utils.pick)(moduleInstance.trackGlobalNamespaces || [], globalInstance.store.getState())]);
          } //create store


          moduleInstance.store = (0, _utils.ensureAncestorRender)((0, _redux.createStore)(moduleInstance.reducer, iniState, composeEnhancers((0, _objectSpread3["default"])({
            name: moduleInstance.namespace,
            serialize: {
              replacer: function replacer(key, value) {
                if (value && value.dispatchConfig && value._targetInst // eslint-disable-line
                ) {
                    return "[EVENT]";
                  }

                return value;
              }
            }
          }, options)).apply(void 0, [_redux.applyMiddleware.apply(void 0, (0, _toConsumableArray2["default"])(middleware))].concat((0, _toConsumableArray2["default"])(moduleInstance.enhancers || [])))));
        } else if (contextId !== _constants.GLOBAL_CONTEXT_ID && (0, _utils.prop)("store", globalInstance)) {
          moduleInstance.store.dispatch((0, _middleware.globalUpdate)({
            type: "SyncGlobalState"
          }, (0, _utils.pick)(moduleInstance.trackGlobalNamespaces || [], globalInstance.store.getState())));
        }

        moduleInstance.store.subscribeExecute();

        if (moduleInstance.bridgeMiddleware) {
          moduleInstance.bridgeMiddleware.bind();
        }

        return moduleInstance;
      },
      componentWillUnmount: function componentWillUnmount(_ref4) {
        var moduleInstances = _ref4.moduleInstances,
            contextId = _ref4.contextId;
        var moduleInstance = moduleInstances[contextId]; //stop middleware

        if (moduleInstance.bridgeMiddleware) {
          moduleInstance.bridgeMiddleware.unbind();
        }

        moduleInstance.store.unsubscribeExecute();
        return moduleInstance;
      }
    });
  };
};

exports.addLifecycle = addLifecycle;

var merge = function merge(left) {
  return function (right) {
    var dependencies = (0, _objectSpread3["default"])({}, left.dependencies, {}, right.dependencies, (0, _defineProperty2["default"])({}, left.namespace, left));
    return (0, _objectSpread3["default"])({}, right, {
      dependencies: dependencies
    });
  };
};

exports.merge = merge;

var makeDispatchToGlobal = function makeDispatchToGlobal(namespaces) {
  var re = new RegExp("(".concat(namespaces.join("|"), ")/"));
  return function (action) {
    return !re.test(action.type);
  };
};

var addBridge = function addBridge() {
  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      trackGlobalNamespaces = _ref5.trackGlobalNamespaces,
      dispatchToGlobal = _ref5.dispatchToGlobal;

  return function (module) {
    return (0, _objectSpread3["default"])({}, module, {
      trackGlobalNamespaces: (0, _utils.uniq)(Object.values(module.dependencies || {}).reduce(function (trackGlobalNamespaces, module) {
        return [].concat((0, _toConsumableArray2["default"])(trackGlobalNamespaces), (0, _toConsumableArray2["default"])(module.trackGlobalNamespaces || []));
      }, Array.isArray(trackGlobalNamespaces) ? trackGlobalNamespaces : [])),
      dispatchToGlobal: typeof dispatchToGlobal === "function" ? dispatchToGlobal : makeDispatchToGlobal((0, _utils.uniq)([].concat((0, _toConsumableArray2["default"])(Object.values(module.dependencies || {}).map(function (val) {
        return val.namespace;
      })), [module.namespace])))
    });
  };
};

exports.addBridge = addBridge;
var createModule = (0, _redux.compose)(addLifecycle(), addReducer());
exports.createModule = createModule;