"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useModel = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _context = require("./context");

var _utils = require("./utils");

var getStateProps = function getStateProps(_ref) {
  var selectors = _ref.selectors,
      instanceId = _ref.instanceId,
      cache = _ref.cache,
      state = _ref.state,
      props = _ref.props;
  return Object.entries(selectors || {}).reduce(function (acc, _ref2) {
    var _ref3 = (0, _slicedToArray2["default"])(_ref2, 2),
        key = _ref3[0],
        f = _ref3[1];

    cache[f.id] = cache[f.id] || {};
    return (0, _objectSpread3["default"])({}, acc, (0, _defineProperty2["default"])({}, key, f(state, (0, _objectSpread3["default"])({}, props || {}, {
      instanceId: instanceId
    }), cache)));
  }, {});
}; // eslint-disable-next-line no-unused-vars


var useModel = function useModel(selectors, actions, _ref4) {
  var children = _ref4.children,
      props = (0, _objectWithoutProperties2["default"])(_ref4, ["children"]);
  var context = (0, _react.useContext)(_context.StoreManager);
  var instanceId = (0, _utils.propOr)(context.instanceId, "instanceId", props);
  var store = (0, _utils.path)(["moduleInstances", context.contextId, "store"], context);

  if (!store && process.env.NODE_ENV !== "production") {
    throw Error("No store found for `useModel`. Please use `createContext` in a parent component.");
  }

  var _useState = (0, _react.useState)({}),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 1),
      cache = _useState2[0];

  var iniStateProps = (0, _react.useMemo)(function () {
    return getStateProps({
      selectors: selectors,
      instanceId: instanceId,
      cache: cache,
      state: store.getState(),
      props: props
    });
  }, []);

  var _useState3 = (0, _react.useState)(iniStateProps),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      stateProps = _useState4[0],
      setStateProps = _useState4[1];

  (0, _react.useEffect)(function () {
    if (store) {
      var oldStateProps = stateProps;
      return store.subscribe(function () {
        var newStateProps = getStateProps({
          selectors: selectors,
          instanceId: instanceId,
          cache: cache,
          state: store.getState(),
          props: props
        });

        if ((0, _utils.diff)(oldStateProps, newStateProps)) {
          oldStateProps = newStateProps;
          setStateProps(newStateProps);
        }
      }, context.renderLevel);
    }

    return _utils.noop;
  }, [store, props, instanceId]);
  var actionProps = (0, _react.useMemo)(function () {
    return Object.entries(actions || {}).reduce(function (actions, _ref5) {
      var _ref6 = (0, _slicedToArray2["default"])(_ref5, 2),
          key = _ref6[0],
          f = _ref6[1];

      if (typeof f !== "function") {
        throw Error("".concat(key, " is not a function"));
      }

      actions[key] = function (payload) {
        var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var error = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
        return store.dispatch(f(payload, (0, _objectSpread3["default"])({}, props, {
          meta: (0, _objectSpread3["default"])({
            instanceId: instanceId
          }, props.meta || {})
        }), error));
      };

      return actions;
    }, {});
  }, [instanceId]);
  return (0, _react.useMemo)(function () {
    return (0, _objectSpread3["default"])({}, stateProps, {}, actionProps, {
      instanceId: instanceId
    });
  }, [stateProps, actionProps, instanceId]);
};

exports.useModel = useModel;