"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeBridgeMiddleware = exports.globalUpdate = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _toPropertyKey2 = _interopRequireDefault(require("@babel/runtime/helpers/toPropertyKey"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread4 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _utils = require("./utils");

var _constants = require("./constants");

var globalUpdate = function globalUpdate(action, state) {
  return (0, _objectSpread4["default"])({}, action, {
    meta: (0, _objectSpread4["default"])({}, action.meta || {}, (0, _defineProperty2["default"])({}, _constants.GLOBAL_UPDATE, state))
  });
};

exports.globalUpdate = globalUpdate;

var makeBridgeMiddleware = function makeBridgeMiddleware(_ref) {
  var moduleInstance = _ref.moduleInstance,
      globalInstance = _ref.globalInstance;
  var middleware = _utils.noop;

  if (moduleInstance !== globalInstance) {
    // eslint-disable-next-line no-unused-vars
    middleware = function middleware(store) {
      return function (next) {
        return function (action) {
          if (moduleInstance.enableDispatchToGlobal && !(0, _utils.has)(["meta", _constants.GLOBAL_UPDATE], action) && moduleInstance.dispatchToGlobal(action)) {
            return globalInstance && globalInstance.store && globalInstance.store.dispatch(action);
          }

          return next(action);
        };
      };
    };
  } else {
    middleware = function middleware(store) {
      var lastState = store.getState();
      return function (next) {
        return function (action) {
          var nextAction = next(action);
          var listeners = Object.values(globalInstance.listeners || {});

          if (listeners.length === 0) {
            return nextAction;
          }

          var nextState = store.getState();

          if ((0, _utils.diff)(lastState, nextState)) {
            listeners.forEach(function (_ref2) {
              var dispatch = _ref2.dispatch,
                  trackGlobalNamespaces = _ref2.trackGlobalNamespaces;

              if ((0, _utils.diff)(lastState, nextState, trackGlobalNamespaces)) {
                dispatch(globalUpdate(action, (0, _utils.pick)(trackGlobalNamespaces, nextState)));
              }
            });
            lastState = nextState;
          }

          return nextAction;
        };
      };
    };
  }

  middleware.bind = function () {
    if (moduleInstance !== globalInstance) {
      moduleInstance.enableDispatchToGlobal = typeof moduleInstance.dispatchToGlobal === "function";

      if (moduleInstance.trackGlobalNamespaces && moduleInstance.trackGlobalNamespaces.length) {
        globalInstance.listeners = (0, _objectSpread4["default"])({}, globalInstance.listeners || {}, (0, _defineProperty2["default"])({}, moduleInstance.namespace, {
          trackGlobalNamespaces: moduleInstance.trackGlobalNamespaces,
          dispatch: moduleInstance.store.dispatch
        }));
      }
    }
  };

  middleware.unbind = function () {
    if (moduleInstance !== globalInstance) {
      moduleInstance.enableDispatchToGlobal = false; // eslint-disable-next-line no-unused-vars

      var _ref3 = globalInstance.listeners || {},
          _moduleInstance$names = moduleInstance.namespace,
          remove = _ref3[_moduleInstance$names],
          listeners = (0, _objectWithoutProperties2["default"])(_ref3, [_moduleInstance$names].map(_toPropertyKey2["default"]));

      globalInstance.listeners = listeners;
    }
  };

  return middleware;
};

exports.makeBridgeMiddleware = makeBridgeMiddleware;